@@ -47,7 +47,6 @@ and IEC958 structure.
 #include <string.h>
 #include <fcntl.h>
 #include <signal.h>
-#include <dlfcn.h>
 #include <sys/poll.h>
 #include "control_local.h"
 
@@ -758,7 +757,7 @@ snd_ctl_t *snd_async_handler_get_ctl(snd_async_handler_t *handler)
 	return handler->u.ctl;
 }
 
-static const char *build_in_ctls[] = {
+static const char *const build_in_ctls[] = {
 	"hw", "shm", NULL
 };
 
@@ -843,20 +842,20 @@ static int snd_ctl_open_conf(snd_ctl_t **ctlp, const char *name,
 		sprintf(buf, "_snd_ctl_%s_open", str);
 	}
 	if (!lib) {
-		const char **build_in = build_in_ctls;
+		const char *const *build_in = build_in_ctls;
 		while (*build_in) {
 			if (!strcmp(*build_in, str))
 				break;
 			build_in++;
 		}
 		if (*build_in == NULL) {
-			buf1 = malloc(strlen(str) + sizeof(PKGLIBDIR) + 32);
+			buf1 = malloc(strlen(str) + sizeof(ALSA_PLUGIN_DIR) + 32);
 			if (buf1 == NULL) {
 				err = -ENOMEM;
 				goto _err;
 			}
 			lib = buf1;
-			sprintf(buf1, "%s/libasound_module_ctl_%s.so", PKGLIBDIR, str);
+			sprintf(buf1, "%s/libasound_module_ctl_%s.so", ALSA_PLUGIN_DIR, str);
 		}
 	}
 #ifndef PIC
@@ -950,9 +949,10 @@ int snd_ctl_open_lconf(snd_ctl_t **ctlp, const char *name,
 #ifndef DOC_HIDDEN
 #define TYPE(v) [SND_CTL_ELEM_TYPE_##v] = #v
 #define IFACE(v) [SND_CTL_ELEM_IFACE_##v] = #v
+#define IFACE1(v, n) [SND_CTL_ELEM_IFACE_##v] = #n
 #define EVENT(v) [SND_CTL_EVENT_##v] = #v
 
-static const char *snd_ctl_elem_type_names[] = {
+static const char *const snd_ctl_elem_type_names[] = {
 	TYPE(NONE),
 	TYPE(BOOLEAN),
 	TYPE(INTEGER),
@@ -962,7 +962,7 @@ static const char *snd_ctl_elem_type_names[] = {
 	TYPE(INTEGER64),
 };
 
-static const char *snd_ctl_elem_iface_names[] = {
+static const char *const snd_ctl_elem_iface_names[] = {
 	IFACE(CARD),
 	IFACE(HWDEP),
 	IFACE(MIXER),
@@ -972,7 +972,7 @@ static const char *snd_ctl_elem_iface_names[] = {
 	IFACE(SEQUENCER),
 };
 
-static const char *snd_ctl_event_type_names[] = {
+static const char *const snd_ctl_event_type_names[] = {
 	EVENT(ELEM),
 };
 #endif
@@ -1876,7 +1876,7 @@ int snd_ctl_elem_info_is_tlv_commandable(const snd_ctl_elem_info_t *obj)
 int snd_ctl_elem_info_is_indirect(const snd_ctl_elem_info_t *obj)
 {
 	assert(obj);
-	return !!(obj->access & SNDRV_CTL_ELEM_ACCESS_INDIRECT);
+	return 0;
 }
 link_warning(snd_ctl_elem_info_is_indirect, "Warning: snd_ctl_elem_info_is_indirect is deprecated, do not use it");
 
@@ -2023,8 +2023,6 @@ int snd_ctl_elem_info_get_dimensions(const snd_ctl_elem_info_t *obj)
 	int i;
 
 	assert(obj);
-	if (obj->access & SNDRV_CTL_ELEM_ACCESS_DINDIRECT)
-		return 0;			/* FIXME: implement indirect access as well */
 	for (i = 3; i >= 0; i--)
 		if (obj->dimen.d[i])
 			break;
@@ -2045,8 +2043,6 @@ int snd_ctl_elem_info_get_dimension(const snd_ctl_elem_info_t *obj, unsigned int
 #endif
 {
 	assert(obj);
-	if (obj->access & SNDRV_CTL_ELEM_ACCESS_DINDIRECT)
-		return 0;			/* FIXME: implement indirect access as well */
 	if (idx >= 3)
 		return 0;
 	return obj->dimen.d[idx];
