@@ -1,7 +1,7 @@
 /**
  * \file control/hcontrol.c
  * \brief HCTL Interface - High Level CTL
- * \author Jaroslav Kysela <perex@suse.cz>
+ * \author Jaroslav Kysela <perex@perex.cz>
  * \author Abramo Bagnara <abramo@alsa-project.org>
  * \date 2000
  *
@@ -11,7 +11,7 @@
  */
 /*
  *  Control Interface - high level API
- *  Copyright (c) 2000 by Jaroslav Kysela <perex@suse.cz>
+ *  Copyright (c) 2000 by Jaroslav Kysela <perex@perex.cz>
  *  Copyright (c) 2001 by Abramo Bagnara <abramo@alsa-project.org>
  *
  *
@@ -48,11 +48,13 @@ to reduce overhead accessing the real controls in kernel drivers.
 #include <string.h>
 #include <fcntl.h>
 #include <sys/ioctl.h>
-#include <pthread.h>
 #ifndef DOC_HIDDEN
 #define __USE_GNU
 #endif
 #include "control_local.h"
+#ifdef HAVE_LIBPTHREAD
+#include <pthread.h>
+#endif
 
 #ifndef DOC_HIDDEN
 #define NOT_FOUND 1000000000
@@ -232,13 +234,14 @@ static int snd_hctl_compare_mixer_priority_lookup(const char **name, const char
 
 static int get_compare_weight(const snd_ctl_elem_id_t *id)
 {
-	static const char *names[] = {
+	static const char *const names[] = {
 		"Master",
 		"Hardware Master",
 		"Headphone",
 		"Tone Control",
 		"3D Control",
 		"PCM",
+		"Front",
 		"Surround",
 		"Center",
 		"LFE",
@@ -267,7 +270,7 @@ static int get_compare_weight(const snd_ctl_elem_id_t *id)
 		"IEC958",
 		NULL
 	};
-	static const char *names1[] = {
+	static const char *const names1[] = {
 		"Switch",
 		"Volume",
 		"Playback",
@@ -281,7 +284,7 @@ static int get_compare_weight(const snd_ctl_elem_id_t *id)
 		"-",
 		NULL
 	};
-	static const char *names2[] = {
+	static const char *const names2[] = {
 		"Switch",
 		"Volume",
 		"Bypass",
@@ -419,17 +422,22 @@ static int hctl_compare(const void *a, const void *b) {
 static void snd_hctl_sort(snd_hctl_t *hctl)
 {
 	unsigned int k;
+#ifdef HAVE_LIBPTHREAD
 	static pthread_mutex_t sync_lock = PTHREAD_MUTEX_INITIALIZER;
+#endif
 
 	assert(hctl);
 	assert(hctl->compare);
 	INIT_LIST_HEAD(&hctl->elems);
 
+#ifdef HAVE_LIBPTHREAD
 	pthread_mutex_lock(&sync_lock);
+#endif
 	compare_hctl = hctl;
 	qsort(hctl->pelems, hctl->count, sizeof(*hctl->pelems), hctl_compare);
+#ifdef HAVE_LIBPTHREAD
 	pthread_mutex_unlock(&sync_lock);
-
+#endif
 	for (k = 0; k < hctl->count; k++)
 		list_add_tail(&hctl->pelems[k]->list, &hctl->elems);
 }
@@ -606,8 +614,7 @@ int snd_hctl_load(snd_hctl_t *hctl)
 	}
 	err = snd_ctl_subscribe_events(hctl->ctl, 1);
  _end:
-	if (list.pids)
-		free(list.pids);
+	free(list.pids);
 	return err;
 }
 
@@ -658,22 +665,52 @@ unsigned int snd_hctl_get_count(snd_hctl_t *hctl)
  * \brief Wait for a HCTL to become ready (i.e. at least one event pending)
  * \param hctl HCTL handle
  * \param timeout maximum time in milliseconds to wait
- * \return 0 otherwise a negative error code on failure
+ * \return a positive value on success otherwise a negative error code
+ * \retval 0 timeout occurred
+ * \retval 1 an event is pending
  */
 int snd_hctl_wait(snd_hctl_t *hctl, int timeout)
 {
-	struct pollfd pfd;
-	int err;
-	err = snd_hctl_poll_descriptors(hctl, &pfd, 1);
+	struct pollfd *pfd;
+	unsigned short *revents;
+	int i, npfds, pollio, err, err_poll;
+	
+	npfds = snd_hctl_poll_descriptors_count(hctl);
+	if (npfds <= 0 || npfds >= 16) {
+		SNDERR("Invalid poll_fds %d\n", npfds);
+		return -EIO;
+	}
+	pfd = alloca(sizeof(*pfd) * npfds);
+	revents = alloca(sizeof(*revents) * npfds);
+	err = snd_hctl_poll_descriptors(hctl, pfd, npfds);
 	if (err < 0)
 		return err;
-	if (! err)
-		return 0;
-	assert(err == 1);
-	err = poll(&pfd, 1, timeout);
-	if (err < 0)
-		return -errno;
-	return 0;
+	if (err != npfds) {
+		SNDMSG("invalid poll descriptors %d\n", err);
+		return -EIO;
+	}
+	do {
+		pollio = 0;
+		err_poll = poll(pfd, npfds, timeout);
+		if (err_poll < 0) {
+			if (errno == EINTR)
+				continue;
+			return -errno;
+		}
+		if (! err_poll)
+			break;
+		err = snd_hctl_poll_descriptors_revents(hctl, pfd, npfds, revents);
+		if (err < 0)
+			return err;
+		for (i = 0; i < npfds; i++) {
+			if (revents[i] & (POLLERR | POLLNVAL))
+				return -EIO;
+			if ((revents[i] & (POLLIN | POLLOUT)) == 0)
+				continue;
+			pollio++;
+		}
+	} while (! pollio);
+	return err_poll > 0 ? 1 : 0;
 }
 
 /**
@@ -792,7 +829,9 @@ int snd_hctl_elem_read(snd_hctl_elem_t *elem, snd_ctl_elem_value_t * value)
  * \brief Set value for an HCTL element
  * \param elem HCTL element
  * \param value HCTL element value
- * \return 0 otherwise a negative error code on failure
+ * \retval 0 on success
+ * \retval >1 on success when value was changed
+ * \retval <0 a negative error code on failure
  */
 int snd_hctl_elem_write(snd_hctl_elem_t *elem, snd_ctl_elem_value_t * value)
 {
@@ -803,6 +842,53 @@ int snd_hctl_elem_write(snd_hctl_elem_t *elem, snd_ctl_elem_value_t * value)
 	return snd_ctl_elem_write(elem->hctl->ctl, value);
 }
 
+/**
+ * \brief Get TLV value for an HCTL element
+ * \param elem HCTL element
+ * \param tlv TLV array for value
+ * \param tlv_size size of TLV array in bytes
+ * \return 0 otherwise a negative error code on failure
+ */
+int snd_hctl_elem_tlv_read(snd_hctl_elem_t *elem, unsigned int *tlv, unsigned int tlv_size)
+{
+	assert(elem);
+	assert(tlv);
+	assert(tlv_size >= 12);
+	return snd_ctl_elem_tlv_read(elem->hctl->ctl, &elem->id, tlv, tlv_size);
+}
+
+/**
+ * \brief Set TLV value for an HCTL element
+ * \param elem HCTL element
+ * \param tlv TLV array for value
+ * \retval 0 on success
+ * \retval >1 on success when value was changed
+ * \retval <0 a negative error code on failure
+ */
+int snd_hctl_elem_tlv_write(snd_hctl_elem_t *elem, const unsigned int *tlv)
+{
+	assert(elem);
+	assert(tlv);
+	assert(tlv[1] >= 4);
+	return snd_ctl_elem_tlv_write(elem->hctl->ctl, &elem->id, tlv);
+}
+
+/**
+ * \brief Set TLV value for an HCTL element
+ * \param elem HCTL element
+ * \param tlv TLV array for value
+ * \retval 0 on success
+ * \retval >1 on success when value was changed
+ * \retval <0 a negative error code on failure
+ */
+int snd_hctl_elem_tlv_command(snd_hctl_elem_t *elem, const unsigned int *tlv)
+{
+	assert(elem);
+	assert(tlv);
+	assert(tlv[1] >= 4);
+	return snd_ctl_elem_tlv_command(elem->hctl->ctl, &elem->id, tlv);
+}
+
 /**
  * \brief Get HCTL handle for an HCTL element
  * \param elem HCTL element
