@@ -78,31 +78,14 @@ static inline int hw_is_interval(snd_pcm_hw_param_t var)
 		var <= SND_PCM_HW_PARAM_LAST_INTERVAL;
 }
 
-static inline snd_mask_t *hw_param_mask(snd_pcm_hw_params_t *params,
-					snd_pcm_hw_param_t var)
-{
-	assert(hw_is_mask(var));
-	return (snd_mask_t*)&params->masks[var - SND_PCM_HW_PARAM_FIRST_MASK];
-}
+#define hw_param_mask(params,var) \
+	&((params)->masks[(var) - SND_PCM_HW_PARAM_FIRST_MASK])
 
-static inline snd_interval_t *hw_param_interval(snd_pcm_hw_params_t *params,
-						snd_pcm_hw_param_t var)
-{
-	assert(hw_is_interval(var));
-	return &params->intervals[var - SND_PCM_HW_PARAM_FIRST_INTERVAL];
-}
+#define hw_param_interval(params,var) \
+	&((params)->intervals[(var) - SND_PCM_HW_PARAM_FIRST_INTERVAL])
 
-static inline const snd_mask_t *hw_param_mask_c(const snd_pcm_hw_params_t *params,
-						snd_pcm_hw_param_t var)
-{
-	return (const snd_mask_t *)hw_param_mask((snd_pcm_hw_params_t*) params, var);
-}
-
-static inline const snd_interval_t *hw_param_interval_c(const snd_pcm_hw_params_t *params,
-							snd_pcm_hw_param_t var)
-{
-	return (const snd_interval_t *)hw_param_interval((snd_pcm_hw_params_t*) params, var);
-}
+#define hw_param_mask_c hw_param_mask
+#define hw_param_interval_c hw_param_interval
 
 static void _snd_pcm_hw_param_any(snd_pcm_hw_params_t *params, snd_pcm_hw_param_t var)
 {
@@ -1249,7 +1232,7 @@ void snd_pcm_hw_param_dump(const snd_pcm_hw_params_t *params,
 
 #define HW_PARAM(v) [SND_PCM_HW_PARAM_##v] = #v
 
-const char *snd_pcm_hw_param_names[] = {
+static const char *const snd_pcm_hw_param_names[] = {
 	HW_PARAM(ACCESS),
 	HW_PARAM(FORMAT),
 	HW_PARAM(SUBFORMAT),
@@ -1677,7 +1660,7 @@ int snd_pcm_hw_params_try_explain_failure(snd_pcm_t *pcm,
 typedef struct _snd_pcm_hw_rule snd_pcm_hw_rule_t;
 
 typedef int (*snd_pcm_hw_rule_func_t)(snd_pcm_hw_params_t *params,
-				      snd_pcm_hw_rule_t *rule);
+				      const snd_pcm_hw_rule_t *rule);
 
 struct _snd_pcm_hw_rule {
 	int var;
@@ -1687,7 +1670,7 @@ struct _snd_pcm_hw_rule {
 };
 
 static int snd_pcm_hw_rule_mul(snd_pcm_hw_params_t *params,
-			       snd_pcm_hw_rule_t *rule)
+			       const snd_pcm_hw_rule_t *rule)
 {
 	snd_interval_t t;
 	snd_interval_mul(hw_param_interval_c(params, rule->deps[0]),
@@ -1696,7 +1679,7 @@ static int snd_pcm_hw_rule_mul(snd_pcm_hw_params_t *params,
 }
 
 static int snd_pcm_hw_rule_div(snd_pcm_hw_params_t *params,
-			snd_pcm_hw_rule_t *rule)
+			const snd_pcm_hw_rule_t *rule)
 {
 	snd_interval_t t;
 	snd_interval_div(hw_param_interval_c(params, rule->deps[0]),
@@ -1705,7 +1688,7 @@ static int snd_pcm_hw_rule_div(snd_pcm_hw_params_t *params,
 }
 
 static int snd_pcm_hw_rule_muldivk(snd_pcm_hw_params_t *params,
-				   snd_pcm_hw_rule_t *rule)
+				   const snd_pcm_hw_rule_t *rule)
 {
 	snd_interval_t t;
 	snd_interval_muldivk(hw_param_interval_c(params, rule->deps[0]),
@@ -1715,7 +1698,7 @@ static int snd_pcm_hw_rule_muldivk(snd_pcm_hw_params_t *params,
 }
 
 static int snd_pcm_hw_rule_mulkdiv(snd_pcm_hw_params_t *params,
-				   snd_pcm_hw_rule_t *rule)
+				   const snd_pcm_hw_rule_t *rule)
 {
 	snd_interval_t t;
 	snd_interval_mulkdiv(hw_param_interval_c(params, rule->deps[0]),
@@ -1725,7 +1708,7 @@ static int snd_pcm_hw_rule_mulkdiv(snd_pcm_hw_params_t *params,
 }
 
 static int snd_pcm_hw_rule_format(snd_pcm_hw_params_t *params,
-				  snd_pcm_hw_rule_t *rule)
+				  const snd_pcm_hw_rule_t *rule)
 {
 	int changed = 0;
 	snd_pcm_format_t k;
@@ -1750,7 +1733,7 @@ static int snd_pcm_hw_rule_format(snd_pcm_hw_params_t *params,
 
 
 static int snd_pcm_hw_rule_sample_bits(snd_pcm_hw_params_t *params,
-				       snd_pcm_hw_rule_t *rule)
+				       const snd_pcm_hw_rule_t *rule)
 {
 	unsigned int min, max;
 	snd_pcm_format_t k;
@@ -1784,7 +1767,7 @@ static int snd_pcm_hw_rule_sample_bits(snd_pcm_hw_params_t *params,
 	return changed;
 }
 
-static snd_pcm_hw_rule_t refine_rules[] = {
+static const snd_pcm_hw_rule_t refine_rules[] = {
 	{
 		.var = SND_PCM_HW_PARAM_FORMAT,
 		.func = snd_pcm_hw_rule_format,
@@ -1928,7 +1911,7 @@ static snd_pcm_hw_rule_t refine_rules[] = {
 
 #define RULES (sizeof(refine_rules) / sizeof(refine_rules[0]))
 
-static snd_mask_t refine_masks[SND_PCM_HW_PARAM_LAST_MASK - SND_PCM_HW_PARAM_FIRST_MASK + 1] = {
+static const snd_mask_t refine_masks[SND_PCM_HW_PARAM_LAST_MASK - SND_PCM_HW_PARAM_FIRST_MASK + 1] = {
 	[SND_PCM_HW_PARAM_ACCESS - SND_PCM_HW_PARAM_FIRST_MASK] = {
 		.bits = { 0x1f },
 	},
@@ -1940,7 +1923,7 @@ static snd_mask_t refine_masks[SND_PCM_HW_PARAM_LAST_MASK - SND_PCM_HW_PARAM_FIR
 	},
 };
   
-static snd_interval_t refine_intervals[SND_PCM_HW_PARAM_LAST_INTERVAL - SND_PCM_HW_PARAM_FIRST_INTERVAL + 1] = {
+static const snd_interval_t refine_intervals[SND_PCM_HW_PARAM_LAST_INTERVAL - SND_PCM_HW_PARAM_FIRST_INTERVAL + 1] = {
 	[SND_PCM_HW_PARAM_SAMPLE_BITS - SND_PCM_HW_PARAM_FIRST_INTERVAL] = {
 		.min = 1, .max = UINT_MAX,
 		.openmin = 0, .openmax = 0, .integer = 1, .empty = 0,
@@ -2039,7 +2022,7 @@ int snd_pcm_hw_refine_soft(snd_pcm_t *pcm ATTRIBUTE_UNUSED, snd_pcm_hw_params_t
 	do {
 		again = 0;
 		for (k = 0; k < RULES; k++) {
-			snd_pcm_hw_rule_t *r = &refine_rules[k];
+			const snd_pcm_hw_rule_t *r = &refine_rules[k];
 			unsigned int d;
 			int doit = 0;
 			for (d = 0; r->deps[d] >= 0; d++) {
@@ -2251,7 +2234,7 @@ static int snd_pcm_sw_params_default(snd_pcm_t *pcm, snd_pcm_sw_params_t *params
 	params->period_step = 1;
 	params->sleep_min = 0;
 	params->avail_min = pcm->period_size;
-	params->xfer_align = pcm->period_size;
+	params->xfer_align = 1;
 	params->start_threshold = 1;
 	params->stop_threshold = pcm->buffer_size;
 	params->silence_threshold = 0;
@@ -2326,7 +2309,6 @@ int _snd_pcm_hw_params(snd_pcm_t *pcm, snd_pcm_hw_params_t *params)
 	INTERNAL(snd_pcm_hw_params_get_period_time)(params, &pcm->period_time, 0);
 	INTERNAL(snd_pcm_hw_params_get_period_size)(params, &pcm->period_size, 0);
 	INTERNAL(snd_pcm_hw_params_get_buffer_size)(params, &pcm->buffer_size);
-	INTERNAL(snd_pcm_hw_params_get_tick_time)(params, &pcm->tick_time, 0);
 	pcm->sample_bits = snd_pcm_format_physical_width(pcm->format);
 	pcm->frame_bits = pcm->sample_bits * pcm->channels;
 	fb = pcm->frame_bits;
