@@ -1,7 +1,7 @@
 /**
  * \file rawmidi/rawmidi.c
  * \brief RawMidi Interface
- * \author Jaroslav Kysela <perex@suse.cz>
+ * \author Jaroslav Kysela <perex@perex.cz>
  * \author Abramo Bagnara <abramo@alsa-project.org>
  * \date 2000-2001
  *
@@ -139,7 +139,6 @@ This example shows open and read/write rawmidi operations.
 #include <stdarg.h>
 #include <unistd.h>
 #include <string.h>
-#include <dlfcn.h>
 #include "rawmidi_local.h"
 
 /**
@@ -341,14 +340,12 @@ int snd_rawmidi_close(snd_rawmidi_t *rawmidi)
 {
 	int err;
   	assert(rawmidi);
-	if ((err = rawmidi->ops->close(rawmidi)) < 0)
-		return err;
-	if (rawmidi->name)
-		free(rawmidi->name);
+	err = rawmidi->ops->close(rawmidi);
+	free(rawmidi->name);
 	if (rawmidi->dl_handle)
 		snd_dlclose(rawmidi->dl_handle);
 	free(rawmidi);
-	return 0;
+	return err;
 }
 
 /**
@@ -989,3 +986,19 @@ ssize_t snd_rawmidi_read(snd_rawmidi_t *rawmidi, void *buffer, size_t size)
 	assert(buffer || size == 0);
 	return (rawmidi->ops->read)(rawmidi, buffer, size);
 }
+
+int snd_rawmidi_conf_generic_id(const char *id)
+{
+	static const char ids[][8] = {
+		"comment",
+		"type",
+		"hint",
+	};
+	unsigned int k;
+
+	for (k = 0; k < sizeof ids / sizeof *ids; ++k) {
+		if (strcmp(id, ids[k]) == 0)
+			return 1;
+	}
+	return 0;
+}
